//--------------------------------------------------------------
// File     : stm32_ub_spi2_dma.h
//--------------------------------------------------------------

//--------------------------------------------------------------
#ifndef __STM32F4_UB_SPI2_DMA_H
#define __STM32F4_UB_SPI2_DMA_H

//--------------------------------------------------------------
// Includes
//--------------------------------------------------------------
#include "stm32f4xx.h"
#include "stm32f4xx_gpio.h"
#include "stm32f4xx_rcc.h"
#include "stm32f4xx_spi.h"
#include "stm32f4xx_dma.h"
#include "misc.h"


//--------------------------------------------------------------
// Defines der Puffer
//--------------------------------------------------------------
#define  SPI2_DMA_BUF_SIZE   50    // Grsse vom TX- und RX-Puffer in Bytes



//--------------------------------------------------------------
// SPI-Mode
//--------------------------------------------------------------
typedef enum {
  SPI_MODE_0_MSB = 0,  // CPOL=0, CPHA=0 (MSB-First)
  SPI_MODE_1_MSB,      // CPOL=0, CPHA=1 (MSB-First)
  SPI_MODE_2_MSB,      // CPOL=1, CPHA=0 (MSB-First)
  SPI_MODE_3_MSB,      // CPOL=1, CPHA=1 (MSB-First)
  SPI_MODE_0_LSB,      // CPOL=0, CPHA=0 (LSB-First)
  SPI_MODE_1_LSB,      // CPOL=0, CPHA=1 (LSB-First)
  SPI_MODE_2_LSB,      // CPOL=1, CPHA=0 (LSB-First)
  SPI_MODE_3_LSB       // CPOL=1, CPHA=1 (LSB-First) 
}SPI2_Mode_t;



//--------------------------------------------------------------
// SPI-Clock
// Grundfrequenz (SPI2)= APB1 (APB1=42MHz)
// Mgliche Vorteiler = 2,4,8,16,32,64,128,256
//--------------------------------------------------------------

//#define SPI2_VORTEILER     SPI_BaudRatePrescaler_2   // Frq = 21 MHz
//#define SPI2_VORTEILER     SPI_BaudRatePrescaler_4   // Frq = 10,5 MHz
//#define SPI2_VORTEILER     SPI_BaudRatePrescaler_8   // Frq = 5.25 MHz
//#define SPI2_VORTEILER     SPI_BaudRatePrescaler_16  // Frq = 2.625 MHz
//#define SPI2_VORTEILER     SPI_BaudRatePrescaler_32  // Frq = 1.3125 MHz
#define SPI2_VORTEILER     SPI_BaudRatePrescaler_64  // Frq = 656.2 kHz
//#define SPI2_VORTEILER     SPI_BaudRatePrescaler_128 // Frq = 328.1 kHz
//#define SPI2_VORTEILER     SPI_BaudRatePrescaler_256 // Frq = 164.0 kHz




//--------------------------------------------------------------
// Struktur eines SPI-Pins
//--------------------------------------------------------------
typedef struct {
  GPIO_TypeDef* PORT;     // Port
  const uint16_t PIN;     // Pin
  const uint32_t CLK;     // Clock
  const uint8_t SOURCE;   // Source
}SPI2_PIN_t; 

//--------------------------------------------------------------
// Struktur vom SPI-Device
//--------------------------------------------------------------
typedef struct {
  SPI2_PIN_t  SCK;        // SCK-Pin
  SPI2_PIN_t  MOSI;       // MOSI-Pin
  SPI2_PIN_t  MISO;       // MISO-Pin
}SPI2_DEV_t;


//--------------------------------------------------------------
// Defines vom DMA (siehe Seite 216+217 vom RM0090)
// SPI2_TX = DMA1, Stream4, Channel0 , SPI2_RX = DMA1, Stream3, Channel0
//--------------------------------------------------------------
#define  SPI_DMA_NR              DMA1
#define  SPI_DMA_CLOCK           RCC_AHB1Periph_DMA1
#define  SPI_DMA_CHANNEL         DMA_Channel_0

#define  SPI2_DMA_TX_STREAM      DMA1_Stream4

#define  SPI2_DMA_RX_STREAM      DMA1_Stream3
#define  SPI2_DMA_RX_IRQ         DMA1_Stream3_IRQn
#define  SPI2_DMA_RX_ISR         DMA1_Stream3_IRQHandler
#define  SPI2_DMA_RX_TCIF        DMA_IT_TCIF3


//--------------------------------------------------------------
// Struktur fr SPI_DMA
//--------------------------------------------------------------
typedef struct {
  uint8_t tx_buffer[SPI2_DMA_BUF_SIZE]; // TX-Puffer
  uint8_t rx_buffer[SPI2_DMA_BUF_SIZE]; // RX-Puffer
}SPI2_DMA_t;
SPI2_DMA_t SPI2_DMA;



//--------------------------------------------------------------
// Status vom SPI-Bus
//--------------------------------------------------------------
typedef enum {
  SPI_READY =0,        // SPI bereit zum senden
  SPI_RUNNING          // SPI sendet noch
}SPI2_Status_t;



//--------------------------------------------------------------
// interne Struktur fr UART_DMA
//--------------------------------------------------------------
typedef struct {
  SPI2_Status_t status;               // status
  uint32_t cnt;                       // anzahl der Daten
  uint8_t tx_temp[SPI2_DMA_BUF_SIZE]; // Zwischenspeicher
  uint8_t rx_temp[SPI2_DMA_BUF_SIZE]; // Zwischenspeicher
}P_SPI2_DMA_t;
P_SPI2_DMA_t P_SPI2_DMA;





//--------------------------------------------------------------
// Globale Funktionen
//--------------------------------------------------------------
ErrorStatus UB_SPI2_DMA_Init(SPI2_Mode_t mode);
ErrorStatus UB_SPI2_DMA_SendBuffer(uint32_t cnt);
uint32_t UB_SPI2_DMA_GetReceivedBytes(void);





//--------------------------------------------------------------
#endif // __STM32F4_UB_SPI2_DMA_H
