//--------------------------------------------------------------
// File     : stm32_ub_spi3_dma.h
//--------------------------------------------------------------

//--------------------------------------------------------------
#ifndef __STM32F4_UB_SPI3_DMA_H
#define __STM32F4_UB_SPI3_DMA_H

//--------------------------------------------------------------
// Includes
//--------------------------------------------------------------
#include "stm32f4xx.h"
#include "stm32f4xx_gpio.h"
#include "stm32f4xx_rcc.h"
#include "stm32f4xx_spi.h"
#include "stm32f4xx_dma.h"
#include "misc.h"


//--------------------------------------------------------------
// Defines der Puffer
//--------------------------------------------------------------
#define  SPI3_DMA_BUF_SIZE   50    // Grsse vom TX- und RX-Puffer in Bytes




//--------------------------------------------------------------
// SPI-Mode
//--------------------------------------------------------------
typedef enum {
  SPI_MODE_0_MSB = 0,  // CPOL=0, CPHA=0 (MSB-First)
  SPI_MODE_1_MSB,      // CPOL=0, CPHA=1 (MSB-First)
  SPI_MODE_2_MSB,      // CPOL=1, CPHA=0 (MSB-First)
  SPI_MODE_3_MSB,      // CPOL=1, CPHA=1 (MSB-First)
  SPI_MODE_0_LSB,      // CPOL=0, CPHA=0 (LSB-First)
  SPI_MODE_1_LSB,      // CPOL=0, CPHA=1 (LSB-First)
  SPI_MODE_2_LSB,      // CPOL=1, CPHA=0 (LSB-First)
  SPI_MODE_3_LSB       // CPOL=1, CPHA=1 (LSB-First) 
}SPI3_Mode_t;



//--------------------------------------------------------------
// SPI-Clock
// Grundfrequenz (SPI3)= APB1 (APB1=42MHz)
// Mgliche Vorteiler = 2,4,8,16,32,64,128,256
//--------------------------------------------------------------

//#define SPI3_VORTEILER     SPI_BaudRatePrescaler_2   // Frq = 21 MHz
//#define SPI3_VORTEILER     SPI_BaudRatePrescaler_4   // Frq = 10,5 MHz
//#define SPI3_VORTEILER     SPI_BaudRatePrescaler_8   // Frq = 5.25 MHz
//#define SPI3_VORTEILER     SPI_BaudRatePrescaler_16  // Frq = 2.625 MHz
//#define SPI3_VORTEILER     SPI_BaudRatePrescaler_32  // Frq = 1.3125 MHz
#define SPI3_VORTEILER     SPI_BaudRatePrescaler_64  // Frq = 656.2 kHz
//#define SPI3_VORTEILER     SPI_BaudRatePrescaler_128 // Frq = 328.1 kHz
//#define SPI3_VORTEILER     SPI_BaudRatePrescaler_256 // Frq = 164.0 kHz




//--------------------------------------------------------------
// Struktur eines SPI-Pins
//--------------------------------------------------------------
typedef struct {
  GPIO_TypeDef* PORT;     // Port
  const uint16_t PIN;     // Pin
  const uint32_t CLK;     // Clock
  const uint8_t SOURCE;   // Source
}SPI3_PIN_t; 

//--------------------------------------------------------------
// Struktur vom SPI-Device
//--------------------------------------------------------------
typedef struct {
  SPI3_PIN_t  SCK;        // SCK-Pin
  SPI3_PIN_t  MOSI;       // MOSI-Pin
  SPI3_PIN_t  MISO;       // MISO-Pin
}SPI3_DEV_t;



//--------------------------------------------------------------
// Defines vom DMA (siehe Seite 216+217 vom RM0090)
// SPI3_TX = DMA1, Stream5, Channel0 , SPI3_RX = DMA1, Stream0, Channel0
// SPI3_TX = DMA1, Stream7, Channel0 , SPI3_RX = DMA1, Stream2, Channel0
//--------------------------------------------------------------
#define  SPI_DMA_NR              DMA1
#define  SPI_DMA_CLOCK           RCC_AHB1Periph_DMA1
#define  SPI_DMA_CHANNEL         DMA_Channel_0

#define  SPI3_DMA_TX_STREAM      DMA1_Stream5

#define  SPI3_DMA_RX_STREAM      DMA1_Stream0
#define  SPI3_DMA_RX_IRQ         DMA1_Stream0_IRQn
#define  SPI3_DMA_RX_ISR         DMA1_Stream0_IRQHandler
#define  SPI3_DMA_RX_TCIF        DMA_IT_TCIF0


//--------------------------------------------------------------
// Struktur fr SPI_DMA
//--------------------------------------------------------------
typedef struct {
  uint8_t tx_buffer[SPI3_DMA_BUF_SIZE]; // TX-Puffer
  uint8_t rx_buffer[SPI3_DMA_BUF_SIZE]; // RX-Puffer
}SPI3_DMA_t;
SPI3_DMA_t SPI3_DMA;



//--------------------------------------------------------------
// Status vom SPI-Bus
//--------------------------------------------------------------
typedef enum {
  SPI_READY =0,        // SPI bereit zum senden
  SPI_RUNNING          // SPI sendet noch
}SPI3_Status_t;



//--------------------------------------------------------------
// interne Struktur fr UART_DMA
//--------------------------------------------------------------
typedef struct {
  SPI3_Status_t status;               // status
  uint32_t cnt;                       // anzahl der Daten
  uint8_t tx_temp[SPI3_DMA_BUF_SIZE]; // Zwischenspeicher
  uint8_t rx_temp[SPI3_DMA_BUF_SIZE]; // Zwischenspeicher
}P_SPI3_DMA_t;
P_SPI3_DMA_t P_SPI3_DMA;



//--------------------------------------------------------------
// Globale Funktionen
//--------------------------------------------------------------
ErrorStatus UB_SPI3_DMA_Init(SPI3_Mode_t mode);
ErrorStatus UB_SPI3_DMA_SendBuffer(uint32_t cnt);
uint32_t UB_SPI3_DMA_GetReceivedBytes(void);





//--------------------------------------------------------------
#endif // __STM32F4_UB_SPI3_DMA_H
